@@ -136,6 +136,7 @@ static struct uwsgi_alarm_instance *uwsgi_alarm_get_instance(char *name) {
 }
 
 
+#ifdef UWSGI_PCRE
 static int uwsgi_alarm_log_add(char *alarms, char *regexp, int negate) {
 
 	struct uwsgi_alarm_log *old_ual = NULL, *ual = uwsgi.alarm_logs;
@@ -162,8 +163,10 @@ static int uwsgi_alarm_log_add(char *alarms, char *regexp, int negate) {
 	char *p = strtok(list, ",");
 	while (p) {
 		struct uwsgi_alarm_instance *uai = uwsgi_alarm_get_instance(p);
-		if (!uai)
+		if (!uai) {
+			free(list);
 			return -1;
+		}
 		struct uwsgi_alarm_ll *old_uall = NULL, *uall = ual->alarms;
 		while (uall) {
 			old_uall = uall;
@@ -180,12 +183,38 @@ static int uwsgi_alarm_log_add(char *alarms, char *regexp, int negate) {
 		}
 		p = strtok(NULL, ",");
 	}
+	free(list);
 	return 0;
 }
+#endif
+
+static void uwsgi_alarm_thread_loop(struct uwsgi_thread *ut) {
+	char *buf = uwsgi_malloc(uwsgi.alarm_msg_size + sizeof(long));
+	for (;;) {
+		int interesting_fd = -1;
+                int ret = event_queue_wait(ut->queue, -1, &interesting_fd);
+		if (ret > 0) {
+			ssize_t len = read(ut->pipe[1], buf, uwsgi.alarm_msg_size + sizeof(long));
+			if (len > (ssize_t)(sizeof(long) + 1)) {
+				size_t msg_size = len - sizeof(long);
+				char *msg = buf + sizeof(long);
+				long ptr = 0;
+				memcpy(&ptr, buf, sizeof(long));
+				struct uwsgi_alarm_instance *uai = (struct uwsgi_alarm_instance *) ptr;
+				if (!uai)
+					break;
+				uwsgi_alarm_run(uai, msg, msg_size);
+			}
+		}
+	}
+	free(buf);
+}
 
 // initialize alarms, instances and log regexps
 void uwsgi_alarms_init() {
 
+	if (!uwsgi.master_process) return;
+
 	// first of all, create instance of alarms
 	struct uwsgi_string_list *usl = uwsgi.alarm_list;
 	while (usl) {
@@ -212,6 +241,9 @@ void uwsgi_alarms_init() {
 		usl = usl->next;
 	}
 
+	if (!uwsgi.alarm_instances) return;
+
+#ifdef UWSGI_PCRE
 	// then map log-alarm
 	usl = uwsgi.alarm_logs_list;
 	while (usl) {
@@ -231,8 +263,20 @@ void uwsgi_alarms_init() {
 
 		usl = usl->next;
 	}
+#endif
 }
 
+void uwsgi_alarm_thread_start() {
+	if (!uwsgi.alarm_instances) return;
+	// start the alarm_thread
+	uwsgi.alarm_thread = uwsgi_thread_new(uwsgi_alarm_thread_loop);
+	if (!uwsgi.alarm_thread) {
+		uwsgi_log("unable to spawn alarm thread\n");
+		exit(1);
+	}
+}
+
+#ifdef UWSGI_PCRE
 // check if a log should raise an alarm
 void uwsgi_alarm_log_check(char *msg, size_t len) {
 	if (!uwsgi_strncmp(msg, len, "[uwsgi-alarm", 12))
@@ -250,6 +294,7 @@ void uwsgi_alarm_log_check(char *msg, size_t len) {
 		ual = ual->next;
 	}
 }
+#endif
 
 // call the alarm func
 void uwsgi_alarm_run(struct uwsgi_alarm_instance *uai, char *msg, size_t len) {
@@ -265,6 +310,7 @@ void uwsgi_alarm_run(struct uwsgi_alarm_instance *uai, char *msg, size_t len) {
 	uai->last_msg_size = len;
 }
 
+#ifdef UWSGI_PCRE
 // call the alarms mapped to a log line
 void uwsgi_alarm_log_run(struct uwsgi_alarm_log *ual, char *msg, size_t len) {
 	struct uwsgi_alarm_ll *uall = ual->alarms;
@@ -273,3 +319,24 @@ void uwsgi_alarm_log_run(struct uwsgi_alarm_log *ual, char *msg, size_t len) {
 		uall = uall->next;
 	}
 }
+#endif
+
+
+// this is the api function workers,mules and whatever you want can call from code
+void uwsgi_alarm_trigger(char *alarm_instance_name, char *msg, size_t len) {
+	if (!uwsgi.alarm_thread) return;
+	if (len > uwsgi.alarm_msg_size) return;
+	struct uwsgi_alarm_instance *uai = uwsgi_alarm_get_instance(alarm_instance_name);
+	if (!uai) return;
+
+	struct iovec iov[2];
+	iov[0].iov_base = &uai;
+	iov[0].iov_len = sizeof(long);
+	iov[1].iov_base = msg;
+	iov[1].iov_len = len;
+
+	// now send the message to the alarm thread
+	if (writev(uwsgi.alarm_thread->pipe[0], iov, 2) != (ssize_t) (len+sizeof(long))) {
+		uwsgi_error("[uwsgi-alarm-error] uwsgi_alarm_trigger()/writev()");
+	}
+}
