@@ -56,6 +56,7 @@ void uwsgi_mule(int id) {
 		// avoid race conditions
 		uwsgi.mules[id - 1].id = id;
 		uwsgi.mules[id - 1].pid = getpid();
+		uwsgi.mypid = uwsgi.mules[id - 1].pid;
 
 		uwsgi_fixup_fds(0, id, NULL);
 
@@ -81,7 +82,7 @@ void uwsgi_mule(int id) {
 			for (i = 0; i < 256; i++) {
 				if (uwsgi.p[i]->mule) {
 					if (uwsgi.p[i]->mule(uwsgi.mules[id - 1].patch) == 1) {
-						// never here
+						// never here ?
 						end_me(1);
 					}
 				}
@@ -209,7 +210,7 @@ void uwsgi_mule_handler() {
 					}
 				}
 				if (!found)
-					uwsgi_log("*** mule %d received a %d bytes message ***\n", uwsgi.muleid, len);
+					uwsgi_log("*** mule %d received a %ld bytes message ***\n", uwsgi.muleid, (long) len);
 			}
 		}
 	}
@@ -405,12 +406,12 @@ void uwsgi_setup_mules_and_farms() {
 		uwsgi.mules = (struct uwsgi_mule *) uwsgi_calloc_shared(sizeof(struct uwsgi_mule) * uwsgi.mules_cnt);
 
 		create_signal_pipe(uwsgi.shared->mule_signal_pipe);
-		create_msg_pipe(uwsgi.shared->mule_queue_pipe, uwsgi.mule_msg_size);
+		create_signal_pipe(uwsgi.shared->mule_queue_pipe);
 
 		for (i = 0; i < uwsgi.mules_cnt; i++) {
 			// create the socket pipe
 			create_signal_pipe(uwsgi.mules[i].signal_pipe);
-			create_msg_pipe(uwsgi.mules[i].queue_pipe, uwsgi.mule_msg_size);
+			create_signal_pipe(uwsgi.mules[i].queue_pipe);
 
 			uwsgi.mules[i].id = i + 1;
 
@@ -436,10 +437,11 @@ void uwsgi_setup_mules_and_farms() {
 			mules_list++;
 
 			strncpy(uwsgi.farms[i].name, farm_value, 0xff);
+			free(farm_value);
 
 			// create the socket pipe
 			create_signal_pipe(uwsgi.farms[i].signal_pipe);
-			create_msg_pipe(uwsgi.farms[i].queue_pipe, uwsgi.mule_msg_size);
+			create_signal_pipe(uwsgi.farms[i].queue_pipe);
 
 			char *p = strtok(mules_list, ",");
 			while (p != NULL) {
