@@ -97,7 +97,6 @@ static void carbon_post_init() {
 
 	if (!u_carbon.last_busyness_values) {
 		u_carbon.last_busyness_values = uwsgi_calloc(sizeof(unsigned long long) * uwsgi.numproc);
-		
 	}
 
 	if (!u_carbon.current_busyness_values) {
@@ -150,7 +149,7 @@ static void carbon_push_stats(int retry_cycle, time_t now) {
 	for (i = 0; i < uwsgi.numproc; i++) {
 		u_carbon.current_busyness_values[i] = uwsgi.workers[i+1].running_time - u_carbon.last_busyness_values[i];
 		u_carbon.last_busyness_values[i] = uwsgi.workers[i+1].running_time;
-		u_carbon.was_busy[i-1] += uwsgi.workers[i+1].busy;
+		u_carbon.was_busy[i-1] += uwsgi_worker_is_busy(i+1);
 	}
 
 	u_carbon.need_retry = 0;
