@@ -19,14 +19,12 @@ struct uwsgi_option uwsgi_perl_options[] = {
 #ifdef PERL_VERSION_STRING
         {"perl-version", no_argument, 0, "print perl version", uwsgi_opt_print, PERL_VERSION_STRING, UWSGI_OPT_IMMEDIATE},
 #endif
-	{"perl-args", required_argument, 0, "add items (space separated) to @ARGV", uwsgi_opt_set_str, &uperl.argv_items, 0},
-	{"perl-arg", required_argument, 0, "add an item to @ARGV", uwsgi_opt_add_string_list, &uperl.argv_item, 0},
+        {"perl-args", required_argument, 0, "add items (space separated) to @ARGV", uwsgi_opt_set_str, &uperl.argv_items, 0},
+        {"perl-arg", required_argument, 0, "add an item to @ARGV", uwsgi_opt_add_string_list, &uperl.argv_item, 0},
         {0, 0, 0, 0, 0, 0, 0},
 
 };
 
-extern struct http_status_codes hsc[];
-
 SV *uwsgi_perl_obj_new(char *class, size_t class_len) {
 
 	SV *newobj;
@@ -74,7 +72,7 @@ SV *uwsgi_perl_call_stream(SV *func) {
 
 	SPAGAIN;
         if(SvTRUE(ERRSV)) {
-                uwsgi_log("[uwsgi-perl error] %s\n", SvPV_nolen(ERRSV));
+                uwsgi_log("[uwsgi-perl error] %s", SvPV_nolen(ERRSV));
         }
         else {
                 ret = SvREFCNT_inc(POPs);
@@ -191,8 +189,8 @@ AV *psgi_call(struct wsgi_request *wsgi_req, SV *psgi_func, SV *env) {
 	SPAGAIN;
 
         if(SvTRUE(ERRSV)) {
-                internal_server_error(wsgi_req, "exception raised");
-                uwsgi_log("[uwsgi-perl error] %s\n", SvPV_nolen(ERRSV));
+                uwsgi_500(wsgi_req);
+                uwsgi_log("[uwsgi-perl error] %s", SvPV_nolen(ERRSV));
         }
 	else {
 		ret = (AV *) SvREFCNT_inc(SvRV(POPs));
@@ -261,18 +259,12 @@ SV *build_psgi_env(struct wsgi_request *wsgi_req) {
 
         if (!hv_store(env, "psgi.run_once", 13, newSViv(0), 0)) goto clear;
 
-#ifdef UWSGI_ASYNC
         if (uwsgi.async > 1) {
                 if (!hv_store(env, "psgi.nonblocking", 16, newSViv(1), 0)) goto clear;
         }
         else {
-#else
                 if (!hv_store(env, "psgi.nonblocking", 16, newSViv(0), 0)) goto clear;
-#endif
-
-#ifdef UWSGI_ASYNC
         }
-#endif
 
         if (!hv_store(env, "psgi.streaming", 14, newSViv(1), 0)) goto clear;
 
@@ -299,7 +291,7 @@ SV *build_psgi_env(struct wsgi_request *wsgi_req) {
 	SV *pi = uwsgi_perl_obj_new("uwsgi::input", 12);
         if (!hv_store(env, "psgi.input", 10, pi, 0)) goto clear;
 	
-	if (!hv_store(env, "psgix.input.buffered", 20, newSViv(wsgi_req->body_as_file), 0)) goto clear;
+	if (!hv_store(env, "psgix.input.buffered", 20, newSViv(uwsgi.post_buffering), 0)) goto clear;
 
 	if (uwsgi.threads > 1) {
 		if (!hv_store(env, "psgix.logger", 12,newRV((SV*) ((SV **)wi->responder1)[wsgi_req->async_id]) ,0)) goto clear;
@@ -333,9 +325,6 @@ clear:
 
 int uwsgi_perl_init(){
 
-
-	struct http_status_codes *http_sc;
-
 	int argc;
 	int i;
 
@@ -379,11 +368,6 @@ int uwsgi_perl_init(){
 
 	PERL_SET_CONTEXT(uperl.main[0]);
 
-	// filling http status codes
-	for (http_sc = hsc; http_sc->message != NULL; http_sc++) {
-		http_sc->message_size = strlen(http_sc->message);
-	}
-
 #ifdef PERL_VERSION_STRING
 	uwsgi_log_initial("initialized Perl %s main interpreter at %p\n", PERL_VERSION_STRING, uperl.main[0]);
 #else
@@ -396,14 +380,13 @@ int uwsgi_perl_init(){
 
 int uwsgi_perl_request(struct wsgi_request *wsgi_req) {
 
-#ifdef UWSGI_ASYNC
 	if (wsgi_req->async_status == UWSGI_AGAIN) {
 		return psgi_response(wsgi_req, wsgi_req->async_placeholder);	
 	}
-#endif
+
 	/* Standard PSGI request */
-	if (!wsgi_req->uh.pktsize) {
-		uwsgi_log("Invalid PSGI request. skip.\n");
+	if (!wsgi_req->uh->pktsize) {
+		uwsgi_log("Empty PSGI request. skip.\n");
 		return -1;
 	}
 
@@ -432,7 +415,8 @@ int uwsgi_perl_request(struct wsgi_request *wsgi_req) {
 		}
 
 		if (wsgi_req->app_id == -1) {
-			internal_server_error(wsgi_req, "Perl application not found");	
+			uwsgi_500(wsgi_req);	
+			uwsgi_log("--- unable to find perl application ---\n");
 			// nothing to clear/free
 			return UWSGI_OK;
 		}
@@ -441,6 +425,7 @@ int uwsgi_perl_request(struct wsgi_request *wsgi_req) {
 	struct uwsgi_app *wi = &uwsgi_apps[wsgi_req->app_id];
 	wi->requests++;
 
+
 	if (uwsgi.threads < 2) {
 		if (((PerlInterpreter **)wi->interpreter)[0] != uperl.main[0]) {
 			PERL_SET_CONTEXT(((PerlInterpreter **)wi->interpreter)[0]);
@@ -458,6 +443,7 @@ int uwsgi_perl_request(struct wsgi_request *wsgi_req) {
 	wsgi_req->async_environ = build_psgi_env(wsgi_req);
 	if (!wsgi_req->async_environ) goto clear;
 
+
 	if (uwsgi.threads > 1) {
 		wsgi_req->async_result = psgi_call(wsgi_req, ((SV **)wi->callable)[wsgi_req->async_id], wsgi_req->async_environ);
 	}
@@ -469,7 +455,7 @@ int uwsgi_perl_request(struct wsgi_request *wsgi_req) {
 	if (SvTYPE((AV *)wsgi_req->async_result) == SVt_PVCV) {
 		SV *stream_result = uwsgi_perl_call_stream((SV*)wsgi_req->async_result);		
 		if (!stream_result) {
-			internal_server_error(wsgi_req, "exception raised");
+			uwsgi_500(wsgi_req);
 		}
 		else {
 			SvREFCNT_dec(stream_result);
@@ -478,13 +464,11 @@ int uwsgi_perl_request(struct wsgi_request *wsgi_req) {
 	}
 
 	while (psgi_response(wsgi_req, wsgi_req->async_result) != UWSGI_OK) {
-#ifdef UWSGI_ASYNC
 		if (uwsgi.async > 1) {
 			FREETMPS;
 			LEAVE;
 			return UWSGI_AGAIN;
 		}
-#endif
 	}
 
 clear2:
@@ -519,7 +503,7 @@ static void psgi_call_cleanup_hook(SV *hook, SV *env) {
 	PUTBACK;
 	call_sv(hook, G_DISCARD);
 	if(SvTRUE(ERRSV)) {
-                uwsgi_log("[uwsgi-perl error] %s\n", SvPV_nolen(ERRSV));
+                uwsgi_log("[uwsgi-perl error] %s", SvPV_nolen(ERRSV));
         }
 	FREETMPS;
 	LEAVE;
@@ -646,12 +630,12 @@ int uwsgi_perl_signal_handler(uint8_t sig, void *handler) {
 
         call_sv( SvRV((SV*)handler), G_DISCARD);
 
+        SPAGAIN;
 	if(SvTRUE(ERRSV)) {
-                uwsgi_log("[uwsgi-perl error] %s\n", SvPV_nolen(ERRSV));
+                uwsgi_log("[uwsgi-perl error] %s", SvPV_nolen(ERRSV));
 		ret = -1;
         }
 
-        SPAGAIN;
         PUTBACK;
         FREETMPS;
         LEAVE;
@@ -664,17 +648,16 @@ void uwsgi_perl_run_hook(SV *hook) {
         ENTER;
         SAVETMPS;
         PUSHMARK(SP);
-        XPUSHs( hook );
         PUTBACK;
 
         call_sv( SvRV(hook), G_DISCARD);
 
+        SPAGAIN;
         if(SvTRUE(ERRSV)) {
-                uwsgi_log("[uwsgi-perl error] %s\n", SvPV_nolen(ERRSV));
+                uwsgi_log("[uwsgi-perl error] %s", SvPV_nolen(ERRSV));
 		return;
         }
 
-        SPAGAIN;
         PUTBACK;
         FREETMPS;
         LEAVE;
@@ -688,15 +671,12 @@ static void uwsgi_perl_atexit() {
                 return;
 
         // if busy do not run atexit hooks
-        if (uwsgi.workers[uwsgi.mywid].busy)
+        if (uwsgi_worker_is_busy(uwsgi.mywid))
                 return;
 
-#ifdef UWSGI_ASYNC
         // managing atexit in async mode is a real pain...skip it for now
         if (uwsgi.async > 1)
                 return;
-#endif
-
 realstuff:
 
 	if (uperl.atexit) {
