@@ -2,19 +2,15 @@
 
 extern struct uwsgi_server uwsgi;
 
-extern struct http_status_codes hsc[];
-
 int psgi_response(struct wsgi_request *wsgi_req, AV *response) {
 
 	SV **status_code, **hitem ;
 	AV *headers, *body =NULL;
-	STRLEN hlen;
-	struct http_status_codes *http_sc;
-	int i,vi, base;
-	char *chitem;
+	STRLEN hlen, hlen2;
+	int i;
+	char *chitem, *chitem2;
 	SV **harakiri;
 
-#ifdef UWSGI_ASYNC
 	if (wsgi_req->async_status == UWSGI_AGAIN) {
 
 		wsgi_req->async_force_again = 0;
@@ -22,7 +18,7 @@ int psgi_response(struct wsgi_request *wsgi_req, AV *response) {
 		wsgi_req->switches++;
                 SV *chunk = uwsgi_perl_obj_call(wsgi_req->async_placeholder, "getline");
 		if (!chunk) {
-			internal_server_error(wsgi_req, "exception raised");
+			uwsgi_500(wsgi_req);
 			return UWSGI_OK;
 		}
 
@@ -51,12 +47,15 @@ int psgi_response(struct wsgi_request *wsgi_req, AV *response) {
 			return UWSGI_OK;
                 }
 
-                wsgi_req->response_size += wsgi_req->socket->proto_write(wsgi_req, chitem, hlen);
+		uwsgi_response_write_body_do(wsgi_req, chitem, hlen);
+		uwsgi_pl_check_write_errors {
+			SvREFCNT_dec(chunk);
+			return UWSGI_OK;
+		}
 		SvREFCNT_dec(chunk);
 
 		return UWSGI_AGAIN;
 	}
-#endif
 
 	if (SvTYPE(response) != SVt_PVAV) {
 		uwsgi_log("invalid PSGI response type\n");
@@ -66,36 +65,9 @@ int psgi_response(struct wsgi_request *wsgi_req, AV *response) {
 	status_code = av_fetch(response, 0, 0);
 	if (!status_code) { uwsgi_log("invalid PSGI status code\n"); return UWSGI_OK;}
 
-        wsgi_req->hvec[0].iov_base = "HTTP/1.1 ";
-        wsgi_req->hvec[0].iov_len = 9;
+	char *status_str = SvPV(*status_code, hlen);
+	if (uwsgi_response_prepare_headers(wsgi_req, status_str, hlen)) return UWSGI_OK;
 
-        wsgi_req->hvec[1].iov_base = SvPV(*status_code, hlen);
-
-        wsgi_req->hvec[1].iov_len = 3;
-
-        wsgi_req->status = atoi(wsgi_req->hvec[1].iov_base);
-
-        wsgi_req->hvec[2].iov_base = " ";
-        wsgi_req->hvec[2].iov_len = 1;
-
-        wsgi_req->hvec[3].iov_len = 0;
-
-        // get the status code
-        for (http_sc = hsc; http_sc->message != NULL; http_sc++) {
-                if (!strncmp(http_sc->key, wsgi_req->hvec[1].iov_base, 3)) {
-                        wsgi_req->hvec[3].iov_base = (char *) http_sc->message;
-                        wsgi_req->hvec[3].iov_len = http_sc->message_size;
-                        break;
-                }
-        }
-
-        if (wsgi_req->hvec[3].iov_len == 0) {
-                wsgi_req->hvec[3].iov_base = "Unknown";
-                wsgi_req->hvec[3].iov_len =  7;
-        }
-
-        wsgi_req->hvec[4].iov_base = "\r\n";
-        wsgi_req->hvec[4].iov_len = 2;
 
         hitem = av_fetch(response, 1, 0);
 	if (!hitem) { uwsgi_log("invalid PSGI headers\n"); return UWSGI_OK;}
@@ -103,54 +75,16 @@ int psgi_response(struct wsgi_request *wsgi_req, AV *response) {
         headers = (AV *) SvRV(*hitem);
 	if (!headers) { uwsgi_log("invalid PSGI headers\n"); return UWSGI_OK;}
 
-        base = 5;
-
-
-        // put them in hvec
+        // generate headers
         for(i=0; i<=av_len(headers); i++) {
-		if (wsgi_req->header_cnt+1 > uwsgi.max_vars) {
-			uwsgi_log("no more space in iovec. consider increasing max-vars...\n");
-			break;
-		}
-                vi = (i*2)+base;
                 hitem = av_fetch(headers,i,0);
                 chitem = SvPV(*hitem, hlen);
-                wsgi_req->hvec[vi].iov_base = chitem; wsgi_req->hvec[vi].iov_len = hlen;
-
-                wsgi_req->hvec[vi+1].iov_base = ": "; wsgi_req->hvec[vi+1].iov_len = 2;
-
                 hitem = av_fetch(headers,i+1,0);
-                chitem = SvPV(*hitem, hlen);
-                wsgi_req->hvec[vi+2].iov_base = chitem; wsgi_req->hvec[vi+2].iov_len = hlen;
-
-                wsgi_req->hvec[vi+3].iov_base = "\r\n"; wsgi_req->hvec[vi+3].iov_len = 2;
-
-                wsgi_req->header_cnt++;
-
-                i++;
+                chitem2 = SvPV(*hitem, hlen2);
+		if (uwsgi_response_add_header(wsgi_req, chitem, hlen, chitem2, hlen2)) return UWSGI_OK;
+		i++;
         }
 
-	int j = (i*2)+base;
-	struct uwsgi_string_list *ah = uwsgi.additional_headers;
-        while(ah) {
-		if (wsgi_req->header_cnt+1 > uwsgi.max_vars) {
-			uwsgi_log("no more space in iovec. consider increasing max-vars...\n");
-			break;
-		}
-                wsgi_req->header_cnt++;
-                wsgi_req->hvec[j].iov_base = ah->value;
-                wsgi_req->hvec[j].iov_len = ah->len;
-                j++;
-                wsgi_req->hvec[j].iov_base = "\r\n";
-                wsgi_req->hvec[j].iov_len = 2;
-                j++;
-                ah = ah->next;
- 	}
-
-        wsgi_req->hvec[j].iov_base = "\r\n"; wsgi_req->hvec[j].iov_len = 2;
-
-        wsgi_req->headers_size += wsgi_req->socket->proto_writev_header(wsgi_req, wsgi_req->hvec, j+1);
-
         hitem = av_fetch(response, 2, 0);
 
 	if (!hitem) {
@@ -170,8 +104,11 @@ int psgi_response(struct wsgi_request *wsgi_req, AV *response) {
 					if (SvTYPE(fn) == SVt_IV && SvIV(fn) >= 0) {
 						wsgi_req->sendfile_fd = SvIV(fn);
 						SvREFCNT_dec(fn);	
-						wsgi_req->response_size += uwsgi_sendfile(wsgi_req);
+						uwsgi_response_sendfile_do(wsgi_req, wsgi_req->sendfile_fd, 0, 0);
 						// no need to close here as perl GC will do the close()
+						uwsgi_pl_check_write_errors {
+							// noop
+						}
 						return UWSGI_OK;
 					}
 					SvREFCNT_dec(fn);	
@@ -181,10 +118,13 @@ int psgi_response(struct wsgi_request *wsgi_req, AV *response) {
 			// check for path method
 			if (uwsgi_perl_obj_can(*hitem, "path", 4)) {
 				SV *p = uwsgi_perl_obj_call(*hitem, "path");
-				wsgi_req->sendfile_fd = open(SvPV_nolen(p), O_RDONLY);
+				int fd = open(SvPV_nolen(p), O_RDONLY);
 				SvREFCNT_dec(p);	
-				wsgi_req->response_size += uwsgi_sendfile(wsgi_req);
-				close(wsgi_req->sendfile_fd);
+				// the following function will close fd
+				uwsgi_response_sendfile_do(wsgi_req, fd, 0, 0);
+				uwsgi_pl_check_write_errors {
+					// noop
+				}
 				return UWSGI_OK;
 			}
 		}
@@ -194,32 +134,33 @@ int psgi_response(struct wsgi_request *wsgi_req, AV *response) {
 			wsgi_req->switches++;
                         SV *chunk = uwsgi_perl_obj_call(*hitem, "getline");
 			if (!chunk) {
-				internal_server_error(wsgi_req, "exception raised");
+				uwsgi_500(wsgi_req);
 				break;
 			}
 
                         chitem = SvPV( chunk, hlen);
-#ifdef UWSGI_ASYNC
 			if (uwsgi.async > 1 && wsgi_req->async_force_again) {
 				SvREFCNT_dec(chunk);
 				wsgi_req->async_status = UWSGI_AGAIN;
 				wsgi_req->async_placeholder = (SV *) *hitem;
 				return UWSGI_AGAIN;
 			}
-#endif
                         if (hlen <= 0) {
 				SvREFCNT_dec(chunk);
                                 break;
                         }
-                        wsgi_req->response_size += wsgi_req->socket->proto_write(wsgi_req, chitem, hlen);
+
+			uwsgi_response_write_body_do(wsgi_req, chitem, hlen);
+			uwsgi_pl_check_write_errors {
+				SvREFCNT_dec(chunk);
+                                break;
+			}
 			SvREFCNT_dec(chunk);
-#ifdef UWSGI_ASYNC
 			if (uwsgi.async > 1) {
 				wsgi_req->async_status = UWSGI_AGAIN;
 				wsgi_req->async_placeholder = (SV *) *hitem;
 				return UWSGI_AGAIN;
 			}
-#endif
                 }
 
 
@@ -236,7 +177,10 @@ int psgi_response(struct wsgi_request *wsgi_req, AV *response) {
                 for(i=0; i<=av_len(body); i++) {
                         hitem = av_fetch(body,i,0);
                         chitem = SvPV(*hitem, hlen);
-                        wsgi_req->response_size += wsgi_req->socket->proto_write(wsgi_req, chitem, hlen);
+			uwsgi_response_write_body_do(wsgi_req, chitem, hlen);
+			uwsgi_pl_check_write_errors {
+				break;
+			}
                 }
 
         }
